<?php
/**
* @file
* Define this Export UI plugin.
* https://drupal.org/node/928026
*/
$plugin = array(
  'schema' => 'skos_explorer_presets',  // As defined in hook_schema().
  'access' => 'administer content',  // Define a permission users must have to access these pages.

  // Define the menu item.
  'menu' => array(
    'menu prefix' => 'admin/config/services',
    'menu item' => 'skos_explorer',
    'menu title' => 'SKOS explorer endpoints',
    'menu description' => 'Administer SKOS explorer presets.',
  ),

  // Define user interface texts.
  'title singular' => t('preset'),
  'title plural' => t('presets'),
  'title singular proper' => t('SKOS explorer preset'),
  'title plural proper' => t('SKOS explorer presets'),

  // Define the names of the functions that provide the add/edit forms.
  'form' => array(
    'settings' => 'skos_explorer_ctools_export_ui_form',
    // 'submit' and 'validate' are also valid callbacks.
  ),
);

/**
* Define the preset add/edit form.
*/
function skos_explorer_ctools_export_ui_form(&$form, &$form_state) {
  $preset = $form_state['item'];

  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#description' => t('The human readable name or description of this preset.'),
    '#default_value' => $preset->description,
    '#required' => TRUE,
  );

  // Add Mymodule's configuration interface.
  $form['endpoint'] = array(
    '#type' => 'textfield',
    '#title' => t('SKOS explorer endpoint'),
    '#description' => t('This is just the simplest possible example of a configuration interface.'),
    '#default_value' => $preset->endpoint,
    '#required' => TRUE,
  );

}

/**
* Implementation of hook_ctools_plugin_api().
*
* Tell CTools that we support the default_mymodule_presets API.
*/
function skos_explorer_ctools_plugin_api($owner, $api) {
  if ($owner == 'skos_explorer' && $api == 'default_skos_explorer_presets') {
    return array('version' => 1);
  }
}

/**
* Implementation of hook_default_mymodule_preset().
*
* Provide a couple of default presets.
*/
function skos_explorer_default_skos_explorer_preset() {
  $export = array();

  $agrovoc = new stdClass;
  $agrovoc->api_version = 1;
  $agrovoc->name = 'AGROVOC';
  $agrovoc->description = 'AGROVOC endpoint';
  $agrovoc->endpoint = 'http://202.45.142.113:10035/repositories/agrovoc';
  $export['AGROVOC'] = $agrovoc;

  $unesco = new stdClass;
  $unesco->api_version = 1;
  $unesco->name = 'UNESCO';
  $unesco->description = 'UNESCO endpoint';
  $unesco->endpoint = 'http://skos.um.es/sparql/index.php';
  $export['UNESCO'] = $unesco;

  $gesis = new stdClass;
  $gesis->api_version = 1;
  $gesis->name = 'GESIS';
  $gesis->description = 'GESIS endpoint';
  $gesis->endpoint = 'http://lod.gesis.org/thesoz/sparql';
  $export['GESIS'] = $gesis;

  return $export;
}
