<?php

/*
 * @file sparql_query.inc
 *
 * Include all module helper functions
 */

class skosExplorer {
  protected $endpoint = NULL;


  function __construct($endpoint) {
    $this->endpoint = $endpoint;
  }


  /**
   * Perform a SPARQL query against the remote endpoint
   *
   * @param string $query SPARQL query
   * @param boolean $rdf Whether to return raw RDF code or not - not yet implemented
   */
  function execute_query($query, $op) {
    $config = array(
      'remote_store_endpoint' => $this->endpoint,
    );

    $store = ARC2::getRemoteStore($config);

    $rows = $store->query($query, 'rows');
    debug($rows);
    if ($rows) {
      if (empty($rows)) {
        $errors[] = t('Remote SPARQL query timed out; please try again.');
        watchdog("sparql_query", 'Remote SPARQL query timed out.');
      }
      else {
        switch ($op) {
          // listings
          case 'list':
            foreach ($rows as $row) {
              $results[] = array(
                'xlabLit' => array('value' => $row['xlablit'], 'lang' => $row['xlablit lang']),
                'uri' => array('value' => $row['uri']),
              );
            }
            break;
          // definition
          case 'def':
            $results[] = array(
              'definition' => array('value' => $rows[0]['definition'], 'lang' => $rows[0]['definition lang']),
              'uri' => array('value' => $rows[0]['uri']),
            );
            break;
            // listings
            case 'match':
              foreach ($rows as $row) {
                $results[] = array(
                  'uri' => array('value' => $row['uri']),
                );
              }
              break;
        }
        return $results;
      }
    }
    return array();
  }

  /**
   * Get terms sparql query by label
   *
   * Builds a SPARQL query for getting related terms
   * for a given term URI and lang
   *
   * @param string $uri
   * @param string $lang
   * @return string
   */
  static function getTermsByLabelStartsType($label, $lang) {
    return
      'PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
      SELECT distinct ?uri ?xlablit
      WHERE {
        ?uri skos:prefLabel ?xlablit .
        FILTER( (regex(str(?xlablit), "^' . $label . '", "i")) && (lang(?xlablit)= "' . $lang . '"))
      }
      ORDER BY DESC(?xlablit)';
  }

  /**
   * Get terms sparql query by label
   *
   * Builds a SPARQL query for getting related terms
   * for a given term URI and lang
   *
   * @param string $uri
   * @param string $lang
   * @return string
   */
  static function getTermsByLabelExactType($label, $lang) {
    return
      'PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
      select distinct ?uri ?xlablit
      where {
        ?uri skos:prefLabel ?xlablit .
        FILTER(str(?xlablit) = "' . $label . '" && lang(?xlablit) = "' . $lang . '")
      }';
  }

/**
 * Get related terms sparql query
 *
 * Builds a SPARQL query for getting related terms
 * for a given term URI and lang
 *
 * @param string $uri
 * @param string $lang
 * @return string
 */

  static function getRelatedTermsByURI($uri, $lang) {
    return
      "PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
      SELECT distinct ?uri ?xlablit
      WHERE {
        ?termuri skos:related ?uri .
        ?uri skos:prefLabel ?xlablit .
        FILTER (str(?termuri) = '" . $uri . "' && (lang(?xlablit)='" . $lang . "') )
      }";
  }

  /**
   * Get related terms sparql query
   *
   * Builds a SPARQL query for getting related terms
   * for a given term URI and lang
   *
   * @param string $uri
   * @param string $lang
   * @return string
   */
  static function getNarrowerTermsByURI($uri, $lang) {
    return
      "PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
      SELECT distinct ?uri ?xlablit
      WHERE {
        ?termuri skos:narrower ?uri .
        ?uri skos:prefLabel ?xlablit .
        FILTER (str(?termuri) = '" . $uri . "' && (lang(?xlablit)='" . $lang . "') )
      }";
  }

  /**
   * Get related terms sparql query
   *
   * Builds a SPARQL query for getting related terms
   * for a given term URI and lang
   *
   * @param string $uri
   * @param string $lang
   * @return string
   */
  static function getBroaderTermsByURI($uri, $lang) {
    return
      "PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
        SELECT distinct ?uri ?xlablit
        WHERE {
        ?termuri skos:broader ?uri .
        ?uri skos:prefLabel ?xlablit .
        FILTER (str(?termuri) = '" . $uri . "' && (lang(?xlablit)='" . $lang . "') )
      }";
  }

  //matched terms

  static function getExactMatchTermsByURI($uri) {
    return
      'PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
       SELECT ?uri
       WHERE {
         ?termuri skos:exactMatch ?uri .
         FILTER(str(?termuri) = "' . $uri . '")
      }';
  }

  /**
   * Get translated labels sparql query
   *
   * Builds a SPARQL query for getting translations
   * of a given term URI and lang
   *
   * @param string $uri
   * @param string $lang
   * @return string
   */
  static function getOtherLangsLabel($uri, $lang) {
    return
      'PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
       SELECT distinct ?uri ?xlablit
       WHERE {
         ?uri skos:prefLabel ?xlablit .
         FILTER ( ( str(?uri) = "' . $uri . '" ) && (lang(?xlablit)!="' . $lang . '") )
      }';
  }

  /**
   * Get term's definition query
   *
   * Builds a SPARQL query for getting term definition
   * for a given term URI and lang
   *
   * @param string $uri
   * @param string $lang
   * @return string
   */
  static function getDefinitionByURI($uri, $lang) {
    return
    'PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
    PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#>
    PREFIX skos:<http://www.w3.org/2004/02/skos/core#>
    SELECT ?uri ?definition
    WHERE {
      ?uri skos:definition ?definitionUri .
      ?definitionUri rdf:value ?definition .
    FILTER (str(?uri) = "' . $uri . '" && (lang(?definition)="' . $lang . '") ) }';
  }

}

