<?php
/*
 * @file sparql_query.inc
 *
 * Include all module helper functions
 */

/**
 * Get alphabet for a given lang
 *
 * helper function to build alphabet for a given lang
 * only english currently supported
 *
 * @param string $lang
 * @return array
 */
function _skos_explorer_get_letters_by_lang($lang='en') {
  switch ($lang) {
    case "ru":
      $result = array(
                  "А",
                  "Б",
                  "В",
                  "Г",
                  "Д",
                  "Е",
                  "Ё",
                  "Ж",
                  "З",
                  "И",
                  "Й",
                  "К",
                  "Л",
                  "М",
                  "Н",
                  "О",
                  "П",
                  "Р",
                  "С",
                  "Т",
                  "У",
                  "Ф",
                  "X",
                  "Ц",
                  "Ч",
                  "Ш",
                  "Щ",
                  "Ъ",
                  "Ы",
                  "Ь",
                  "Э",
                  "Ю",
                  "Я"
                );
      break;
    case "ar":
      $result = array(
                  "ا",
                  "ب",
                  "ت",
                  "ث",
                  "ج",
                  "ح",
                  "خ",
                  "د",
                  "ذ",
                  "ر",
                  "ز",
                  "س",
                  "ش",
                  "ص",
                  "ض",
                  "ط",
                  "ظ",
                  "ع",
                  "غ",
                  "ف",
                  "ق",
                  "ك",
                  "ل",
                  "م",
                  "ن",
                  "ه",
                  "و",
                  "ي"
      );
      break;
    case 'zh': $result = array();
      break;
    default:
      $result = array(
                  "A",
                  "B",
                  "C",
                  "D",
                  "E",
                  "F",
                  "G",
                  "H",
                  "I",
                  "J",
                  "K",
                  "L",
                  "M",
                  "N",
                  "O",
                  "P",
                  "Q",
                  "R",
                  "S",
                  "T",
                  "U",
                  "V",
                  "W",
                  "X",
                  "Y",
                  "Z",
                );
      break;
  }

  return $result;

}


/**
 * Return an array of available interface languages
 *
 * @return array
 */
function _skos_explorer_get_available_langs() {
  return array(
    'ar' => t('Arabic'),
    'zh' => t('Chinese'),
    'en' => t('English'),
    'fr' => t('French'),
    'ru' => t('Russian'),
    'es' => t('Spanish'),
  );
}

/**
 * Return extended name of a language, given its code
 *
 * @param string $id
 * @return string
 */
function _skos_explorer_get_lang_from_id($id) {
  $langs = _skos_explorer_get_available_langs();
  if (isset($langs[$id])) {
    return $langs[$id];
  }
  else {
    return;
  }
}

/**
 * Create the alphabetical navigation menu
 *
 * @return string
 */
function _skos_explorer_letters_menu($lang_code = "en") {
  $items = _skos_explorer_get_letters_by_lang($lang_code);

  if ( !empty($items) ) {
    $markup = "<table id='letters-menu'><tr>";

    foreach ($items as $item) {
        $markup .= '<td>' . l($item, "", array("attributes" => array("onclick" => "return Drupal.skos_explorer.showSparqlResultByLetter('" . $item . "')"))) . '</td>';
    }

    $markup .= "</tr></table>";

    return $markup;
  }
  else {
    return '';
  }
}


function _skos_explorer_getDataByLabelStartsType($input, $lang) {
  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getTermsByLabelStartsType($input, $lang);
  $data = $object->execute_query($sparql_query, 'list');

  $items = array();

  foreach ($data as $item) {
    $items[] = _sparql_format_term($item);
  }

  $related_terms = '';

  if (sizeof($items) > 0) {
    $related_terms_structure = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );
    $skos_explorer_debug = variable_get('skos_explorer_debug');
    print '<div class="searchTerms"><div><h3>' . t("Search  results") . '</h3></div>' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . theme_item_list($related_terms_structure) . '</div>';
  }
  exit();
}

function _skos_explorer_getDataByLabelExactType($input, $lang) {
  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getTermsByLabelExactType($input, $lang);
  $data = $object->execute_query($sparql_query, 'list');

  $items = array();

  foreach ($data as $item) {
    $items[] = _sparql_format_term($item);
  }

  $related_terms = '';

  if (sizeof($items) > 0) {
    $related_terms_structure = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );
    $skos_explorer_debug = variable_get('skos_explorer_debug');
    print '<div class="searchTerms"><div><h3>' . t("Search  results") . '</h3></div>' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . theme_item_list($related_terms_structure) . '</div>';
  }
  exit();
}

/**
 * Make a query to the Stellato's webservice to get "starting with" terms
 *
 * Returns a query for terms starting with a given string
 * currently bound to Stellato's webservice, it return the
 * actual result of the query
 *
 * @param string $input string to search for
 * @param string $lang language code for the query
 * @param integer $limit maximum number of results (currently unused)
 *
 * @return array
 *
 * @todo NOT YET SUPPORTED BY WEB SERVICE!
 */
function _skos_explorer_getStartsWithQuery($input, $lang, $limit = 0) {
  return _skos_explorer_getDataByLabelStartsType("$input", $lang);

}

/**
 * Make a query to the Stellato's webservice to get "ending with" terms
 *
 * Returns a query for terms ending with a given string
 * currently bound to Stellato's webservice, it returns the
 * actual result of the query
 *
 * @param string $input string to search for
 * @param string $lang language code for the query
 * @param integer $limit maximum number of results (currently unused)
 *
 * @return array
 *
 * @todo NOT YET SUPPORTED BY WEB SERVICE!
 *  */
function _skos_explorer_getEndsWithQuery($input, $lang, $limit = 0) {
  return _skos_explorer_getDataFromWS("$input*", $lang);
}

/**
 * Make a query to the Stellato's webservice to get terms matching a given string
 *
 * Returns a query for terms matching a given string
 * currently bound to Stellato's webservice, it returns the
 * actual result of the query
 *
 * @param string $input string to search for
 * @param string $lang language code for the query
 * @param integer $limit maximum number of results (currently unused)
 *
 * @return array
 */
function _skos_explorer_getExactMatchQuery($input, $lang, $limit = 0) {
  return _skos_explorer_getDataByLabelExactType("$input", $lang);
}

/**
 * Make a query to the Stellato's webservice to get terms containing a given string
 *
 * Returns a query for terms containing with a given string
 * currently bound to Stellato's webservice, it returns the
 * actual result of the query
 *
 * @param string $input string to search for
 * @param string $lang language code for the query
 * @param integer $limit maximum number of results (currently unused)
 *
 * @return array
 */
function _skos_explorer_getContainsQuery($input, $lang, $limit = 0) {
  return _skos_explorer_getDataFromWS($input, $lang);
}

/**
 * Get a Sparql query for terms with a given string in the definition
 *
 * Returns a SPARQL query for terms matching input parameter
 * in definitions
 *
 * @param string $input string to search for
 * @param string $lang language code for the query
 * @param integer $limit maximum number of results
 *
 * @return string
 *
 */
function _skos_explorer_getSearchByDefinition($input, $lang, $limit = 0) {
  $query = sparql_query_getSearchQuery( "regex(str(?definition), '$input', 'i')", $lang, $limit, $order);
  return $query;
}

/**
 * Set the SPARQL alphabet
 *
 * Prints SPARQL requested alphabet for letter search
 */
function _skos_explorer_ajax_alphabet() {
  $query = trim( $_GET["query"] );
  $lang = $_GET['lang'];

  print _skos_explorer_letters_menu($lang);
  exit();
}


/**
 * Get the SPARQL query formatted result
 *
 * Prints SPARQL query results, to be queried from JS interface layer
 * via AJAX call
 *
 * @param boolean $rdf Whether to return raw RDF code or not - not yet implemented
 *
 */
function _skos_explorer_ajax_result($rdf = FALSE) {
  $query = trim( $_GET["query"] );
  $lang = $_GET['lang'];
  $matchtype = empty($_GET['matchtype']) ? 'starts' : $_GET['matchtype'];
  $endpoint = $_GET['endpoints'];
  variable_set('skos_explorer_endpoint', $endpoint);
  $debug = $_GET['debug'];
  variable_set('skos_explorer_debug', $debug);


  $output = '';
  //choose which query needs to be executed
  switch ($matchtype) {
    case 'exact':
      $result_data = _skos_explorer_getExactMatchQuery($query, $lang, $limit);
      break;
    case 'starts':
      $result_data = _skos_explorer_getStartsWithQuery($query, $lang, $limit);
      break;
    case 'ends':
      $result_data = _skos_explorer_getEndsWithQuery($query, $lang, $limit);
      break;
    case 'contains':
      $result_data = _skos_explorer_getContainsQuery($query, $lang, $limit);
      break;
  }

  $start_time = microtime(true);
  $query_time = (microtime(true)-$start_time);
  $items = array();

  for ($i=0; $i<count($result_data); $i++) {
    $label = $result_data[$i]['xlabLit']['value'];
    if (strcasecmp($label, $query) == 0) {
      //exact match
      $prefix = '0';
    }
    elseif (mb_stripos($label, $query) == 0 ) {
      // starts with
      $prefix = '1';
    }
    else {
      $prefix = '2';
    }
    $result_data[$i]['sortKey'] = $prefix . $label . mb_strpos($label, $query);
  }
  //sort results
  usort($result_data, "_skos_explorer_items_compare");
  //search by letter

  if (in_array($query, _skos_explorer_get_letters_by_lang($lang))) {
    foreach ($result_data as $term) {
      //add only items with first letter matching
      if ( mb_stripos($term['xlabLit']['value'] , $query)  == 0  ) {
        $items[] = _sparql_format_term($term);
      }
    }
  }
  else {
    //format results
    foreach ($result_data as $term) {
      $items[] = _sparql_format_term($term);
    }
  }

  if (sizeof($items) > 0) {
    $variables = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );

    $result = '<h3>' . t("Search Results") . '</h3>' . theme_item_list($variables);
  }
  else {
    $result = '<h3>' . t("No matches") . '</h3>';
  }

  $local_time = (microtime(true)-$start_time-$query_time);

  $output .= '<div class="sparql-results">' . $result . '</div>';

  if (variable_get("sparql_query_debug", FALSE)) {
    $output .= '<em>Remote query took ' . ($query_time*1000) . ' ms,<br /> local processing took ' . ($local_time*1000) . ' ms</em>';
    $output .= '<div class="sparql-code hidden"><pre>' . $sparql_query . '</pre></div>';
  }

  print $output;
  exit();
}

/**
 * Format a single term object
 *
 * @param object $term
 * @return string
 */
function _sparql_format_term($term) {

 //$termLabel = htmlentities($term['xlabLit']['value'],ENT_QUOTES,"UTF-8");

  $termLabel = addslashes(htmlspecialchars($term['xlabLit']['value']));

  return '<a onclick="Drupal.skos_explorer.showSparqlTerm(\'' . $term['uri']['value'] . '\', \'' . $term['xlabLit']['lang'] . '\', \'' . $termLabel . '\')">' . htmlspecialchars( $term['xlabLit']['value'] ) . '</a>';
}


/**
 * Format a single term link object
 *
 * @param object $term
 * @return string
 */
function _sparql_format_term_link($term) {

  return '<a href="' . htmlspecialchars($term['uri']['value']) . '" target="_blank" >' . htmlspecialchars($term['uri']['value']) . '</a>';
}

/**
 * Get the Related terms for a given term markup for a given uri
 *
 * Prints related terms for a given term URI and language
 * parameters are passed via HTTP get
 */
function _skos_explorer_ajax_get_relatedterms_by_uri() {
  $uri = $_REQUEST['uri'];
  $lang = $_REQUEST['lang'];
  //get the sparql query and execute it
  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getRelatedTermsByURI($uri, $lang);
  $data = $object->execute_query($sparql_query, 'list');

  $items = array();

  foreach ($data as $item) {
    $items[] = _sparql_format_term($item);
  }

  $related_terms = '';

  if (sizeof($items) > 0) {
    $related_terms_structure = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );
    $skos_explorer_debug = variable_get('skos_explorer_debug');
    print '<div class="relatedTerms"><div><h3>' . t("Related terms") . '</h3></div>' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . theme_item_list($related_terms_structure) . '</div>';
  }

  exit();
}

function _skos_explorer_ajax_get_broaderterms_by_uri() {
  $uri = $_REQUEST['uri'];
  $lang = $_REQUEST['lang'];
  //get the sparql query and execute it

  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getBroaderTermsByURI($uri, $lang);
  $data = $object->execute_query($sparql_query, 'list');

  $items = array();

  foreach ($data as $item) {
    $items[] = _sparql_format_term($item);
  }

  $related_terms = '';

  if (sizeof($items) > 0) {
    $related_terms_structure = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );

    $skos_explorer_debug = variable_get('skos_explorer_debug');
    print '<div class="broaderTerms"><div><h3>' . t("Broader terms") . '</h3></div>' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . theme_item_list($related_terms_structure) . '</div>';

  }

  exit();
}

function _skos_explorer_ajax_get_narrowerterms_by_uri() {
  $uri = $_REQUEST['uri'];
  $lang = $_REQUEST['lang'];

  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getNarrowerTermsByURI($uri, $lang);
  $data = $object->execute_query($sparql_query, 'list');


  $items = array();

  foreach ($data as $item) {
    $items[] = _sparql_format_term($item);
  }

  $related_terms = '';

  if (sizeof($items) > 0) {
    $related_terms_structure = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );

    $skos_explorer_debug = variable_get('skos_explorer_debug');
    print '<div class="narrowerTerms"><div><h3>' . t("Narrower terms") . '</h3></div>' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . theme_item_list($related_terms_structure) . '</div>';

  }

  exit();
}


/**
 * Get related terms for a given term uri and language
 *
 * Prints terms details for a given term URI and language
 * parameters are passed via HTTP get
 */
function _skos_explorer_ajax_get_term_by_uri() {
  $uri = $_REQUEST['uri'];
  $lang = $_REQUEST['lang'];

  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getDefinitionByURI($uri, $lang);

  $data = $object->execute_query($sparql_query, 'def');

  $row = $data[0];

  $skos_explorer_debug = variable_get('skos_explorer_debug');
  $output = ($skos_explorer_debug == 1 ? '<div class="termDefinition">' . _skos_explorer_debug_display_query($endpoint, $sparql_query) . '</div>' : '' );


  $output .= '<span class="definition">' . $row["definition"]['value'];

  $output .= (isset($row["lastUpdated"]['value'])) ? '<span class="lastUpdated">last updated on ' . date('j F Y', strtotime($row["lastUpdated"]['value'])) . '</span></span>' : '</span>';

  $output .= '<span class="uri"><em>uri:</em> <a target="_blank" href="' . $uri . '">' . $uri . '</a></span>';

  print $output;
  exit();
}

/**
 * Get the other available languages label translations
 *
 * Prints translations of a given term URI and language
 * parameters are passed via HTTP get
 */
function _skos_explorer_ajax_get_otherlang_by_uri() {
  $uri = $_REQUEST['uri'];
  $lang = $_REQUEST['lang'];

  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getOtherLangsLabel($uri, $lang);

  $data = $object->execute_query($sparql_query, 'list');

  $items = array();
  foreach ($data as $item) {
    if (!is_null(_skos_explorer_get_lang_from_id($item['xlabLit']['lang']))) {
      $term = t("Corresponding %lang term", array("%lang" => _skos_explorer_get_lang_from_id($item['xlabLit']['lang'])));
      $items[] = array(
        "term" => $term,
        "data" => _sparql_format_term($item)
      );

    }
  }

  $other_langs_terms_structure = array(
    'items' => $items,
    'type' => "dl",
    'title' => NULL,
    'attributes' => array()
  );


  $skos_explorer_debug = variable_get('skos_explorer_debug');
  print '<div class="preflabels">' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . skos_explorer_definition_list($other_langs_terms_structure) . '</div>';
  exit();
}


function _skos_explorer_ajax_get_exactmatchterms_by_uri() {
  $uri = $_REQUEST['uri'];
  $lang = $_REQUEST['lang'];

  //get the sparql query and execute it
  $endpoint = variable_get('skos_explorer_endpoint', '');
  $object = new skosExplorer($endpoint);
  $sparql_query = $object->getExactMatchTermsByURI($uri, $lang);
  $data = $object->execute_query($sparql_query, 'match');
  $items = array();

  foreach ($data as $item) {
    $items[] = _sparql_format_term_link($item);
  }

  $related_terms = '';

  if (sizeof($items) > 0) {
    $related_terms_structure = array(
      'items' => $items,
      'type' => "ul",
      'title' => NULL,
      'attributes' => array()
    );

    $skos_explorer_debug = variable_get('skos_explorer_debug');
    print '<div class="exactMatchTerms"><div><h3>' . t("External Links") . '</h3></div>' . ( $skos_explorer_debug == 1 ? _skos_explorer_debug_display_query($endpoint, $sparql_query) : '' ) . theme_item_list($related_terms_structure) . '</div>';

  }

  //watchdog(WATCHDOG_DEBUG, "SPARQL: get_relatedterms_by_uri: \n" . $sparql_query);
  exit();
}


function _skos_explorer_debug_display_query($endpoint, $sparql_query) {
return   '<div class="sparql-debug"><div class="sparql-endpoint"><strong>Endpoint:</strong><br />' . $endpoint . '</div><div class="sparql-query"><strong>SPARQL query:</strong><br />' . nl2br(htmlentities($sparql_query)) . '</div>' . theme_item_list($related_terms_structure) . '</div></div>';
}

